 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#ifndef ALGORITHM_H
#define ALGORITHM_H

#include <stdio.h>
#include <iostream>

#include "Instances.h"
#include "Statistics.h"
#include "ObjectiveFunction.h"

using namespace std;

/* This class defines the abstract class Algorithm, which is the superclass of each algorithm object inside 
 * Evoptool. Within this class there are the implementation of the fitness caching mechanism, and operations
 * related to the objective function. It defines several virtual method that must be implemented within 
 * concrete algorithm objects. */

class Algorithm {

private: 
	ObjectiveFunction* _of;				/* Pointer to the Objective Function to solve.			   */
	unsigned int fitnessCalls_;			/* Counter of fitness calls. 				   	   */
	
protected:
	unsigned int objectiveFunctionSize_;		/* Size or length of Objective Function.			   */
	
public:
        Algorithm();
        virtual ~Algorithm();
	
	virtual Statistics* computeStatistics() = 0;
	virtual void printPopulation() = 0;
	virtual void run() = 0;
	virtual string getName() = 0;
	
	virtual void setObjectiveFunction(ObjectiveFunction* of);
	virtual void setObjectiveFunction(const Algorithm &algo);
	double fitness(BinaryInstance *instance);
	unsigned int fitnessCalls();
};

#endif
