
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "Clique.h"
#include <sstream>

/* Class empty costructor. */
Clique::Clique() {

}

/* Class copy costructor. */
Clique::Clique(Clique *copy) {
	_variable = copy->_variable;
}

/* Class destructor. */
Clique::~Clique() {
	_variable.clear();
}

/* Adds a new variable index to this clique. If the index is already present in the clique returns false, 
 * otherwise add the index and return true. */        
bool Clique::add(unsigned int variable) {
	pair<set<unsigned int, ClassCompare>::iterator,bool> ret;
	ret = _variable.insert(variable);
	return ret.second;
}

/* Erases the variable index from the clique. If the index is not present return false, otherwise remove 
 * the index and return true. */
bool Clique::remove(unsigned int variable) {
	return _variable.erase(variable) == 1 ? true : false;
}

/* Searches for the variable index in the clique. Returns true if the index is found, otherwise returns 
 * false. */
bool Clique::find(unsigned int variable) {
	return _variable.count(variable) == 1 ? true : false; 
}

/* Given an individual instance, computes the harmonical value for this clique as the product of
 * individual's values following the harmonic encoding. */
int Clique::harmonicalValue(BinaryInstance *instance) {
	int value = 1; 
	set<unsigned int, ClassCompare>::iterator it; 
	
	for (it = _variable.begin(); it != _variable.end(); it++)
		value *= instance->getHarmonicEncoding(*it);
	return value; 
}

/* Returns the clique size. */
int Clique::size() {
	return _variable.size();
}

/* Returns a string representation of the clique. */
string Clique::toString() {
	stringstream s;
	set<unsigned int, ClassCompare>::iterator it;
	
	s << "<";
	for ( it=_variable.begin() ; it != _variable.end(); it++)
    		s << *it << ",";
    	s << ">";
	
	return s.str();
}

/* Returns the clique as integer array. */
int* Clique::toArray() {
	int i = 0, *variables = new int[_variable.size()];
	set<unsigned int, ClassCompare>::iterator it; 
	
	for (it = _variable.begin(); it != _variable.end(); it++) {
		variables[i] = *it;
		i++; 
	}
		
	return variables; 
}
