
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef CLIQUE_H
#define CLIQUE_H

#include <set>
#include "BinaryInstance.h"

/* This class defines a representation for a clique. A clique is a set of fully connected vertex within a MRF.*/

/* Clique properties derived from set container: 
 * - Unique element values: no two elements in the set can compare equal to each other.
 * - The element value is the key itself.
 * - Elements follow a strict weak ordering at all times.
 */

using namespace std;

struct ClassCompare {
  bool operator() (const int& lhs, const int& rhs) const { return lhs<rhs; }
};

class Clique {
	set<unsigned int, ClassCompare> _variable; 	/* Variable indexes container representing the clique.		 */
public:
	Clique();
	Clique(Clique *copy);
        virtual ~Clique();
        
        bool add(unsigned int variable);
        bool remove(unsigned int variable);
	bool find(unsigned int variable);
	
	int size();
	int* toArray();
	string toString();
	int harmonicalValue(BinaryInstance *instance);

};
#endif
