
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#ifndef PERMUTATIONINSTANCE_H
#define PERMUTATIONINSTANCE_H

#include "BinaryInstance.h"

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_permutation.h>

using namespace std;

/* This class defines the PermutationInstance, which is a mask to be applied on a BinaryInstance. The class 
 * implements several utility functions. */

class PermutationInstance {

private: 
	gsl_permutation *instance_; 
	double fitness_cache_;
	bool valid_cache_;
	
	
public: 
	PermutationInstance(int size);
        PermutationInstance(int size, bool randomized, const gsl_rng *r);
	PermutationInstance(const PermutationInstance &pi);
        virtual ~PermutationInstance();

	int get(unsigned int index);
	void set(unsigned int index, unsigned int value);
	int getSize();
	
	void swap(unsigned int idx1, unsigned int idx2);
	BinaryInstance * applyPermutation(BinaryInstance* instance);
	bool isValid();
	
	void setFitnessCache(double fitness);
	double getFitnessCache();
	bool validCache();
	
private: 
	gsl_permutation * getPermutation();

};

#endif
