
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "SVD.h"

/* Computes the singular value decomposition, returning the U matrix in A. 
 * Normaly U is nr*nr, but if nr>nc only the first nc columns are returned.
 */
void SVD::svd(double **a, double *w, double **v, int rows, int cols) {
        int i,its,j,jj,k,l=0,nm=0;
        bool flag;
        int m = rows;
        int n = cols;
        double c,f,h,s,x,y,z;
        double anorm = 0., g = 0., scale=0. ;
        double *rv1 = new double[n];
       
        for (i = 0; i<n; i++) {
            l = i+1;
            rv1[i] = scale*g;
            g = s = scale  = 0. ;
            if  (i<m)
            {
                for  (k = i; k<m; k++)  scale += SVD::abs(a[k][i]) ;
                if (scale!=0.0)
                {
                    for (k = i; k<m; k++)
                    {
                        a[k][i] /= scale;
                        s += a[k][i]*a[k][i] ;
                    }
                    f = a[i][i];
                    g = -sign(sqrt(s),f) ;
                    h=f*g-s;
                    a[i][i]=f-g;
                    for (j = l; j<n; j++)
                    {
                        for (s = 0,k = i; k<m; k++)
                            s += a[k][i]*a[k][j];
                        f = s/h;
                        for (k = i; k<m; k++)
                            a[k][j] += f*a[k][i];
                    }
                    for (k = i; k<m; k++) a[k][i] *= scale;
                }
            }
            w[i] = scale*g;
            g = s = scale = 0.0 ;
            if (i<m && i!=n-1)
            {	
                for (k = l; k<n; k++)
                    scale += abs(a[i][k]) ;
                if  (scale != 0.)
                {
                    for  (k = l; k<n; k++)
                    {	
                        a[i][k]  /= scale;
                        s += a[i][k]*a[i][k] ;
                    }
                    f = a[i][l];
                    g = -sign(sqrt(s),f);
                    h = f*g-s;
                    a[i][l] = f-g;
                    for  (k = l; k<n; k++)
                        rv1[k] = a[i][k]/h;
                    if (i!=m-1)
                    {		
                        for (j = l; j<m; j++)
                        {	
                            for (s = 0, k = l; k<n; k++)
                                s += a[j][k]*a[i][k] ;
                            for (k = l; k<n; k++)
                                a[j][k] += s*rv1[k] ;
                        }
                    }
                    for (k = l; k<n; k++)
                        a[i][k] *= scale;
                }
            } 
            anorm = anorm > (abs(w[i])+abs(rv1[i])) ? anorm : (abs(w[i])+abs(rv1[i]));
        } 
        
        // NaN Occurs in following loop:
        
        for (i = n-1; i>=0; --i)
        {
            if (i<n-1)
            {	
            	if (g != 0.)
                {
                    for (j = l; j<n; j++)
                    {
                        v[j][i] = (a[i][j]/a[i][l])/g;
                    }
                    
                    //if (SBUtils.ArrayTools.isInfinite(v)) throw new Exception("Infinity found in SVD");//System.exit(3);
                    //if (SBUtils.ArrayTools.isNaN(v)) throw new Exception("NaN found in SVD");//System.exit(3);
                    
                    for (j = l; j<n; j++)
                    {
                        for (s = 0,k = l; k<n; k++)
                        {
                            s += a[i][k]*v[k][j];
                        }
                        
                        for (k = l; k<n; k++)
                        {
                            v[k][j] += s*v[k][i];
                        }
                    }
                }
                for (j = l; j<n; j++)		
                    v[i][j] = v[j][i] = 0.0;
            }
            v[i][i] = 1.0;
            g = rv1[i];
            l = i;
        }
        
        for (i = (m-1) < (n-1) ? (m-1) : (n-1); i>=0; --i)
        {
            l = i+1;
            g = w[i];
            if (i<n-1)			
                for (j = l; j<n; j++)		
                    a[i][j] = 0.0;
            if (g != 0.)
            {
                g = 1./g;
                if (i!= n-1)
                {
                    for(j = l; j<n; j++)
                    {
                        for (s = 0, k = l; k<m; k++)
                            s += a[k][i]*a[k][j];
                        f = (s/a[i][i])*g;
                        for (k = i; k<m; k++)
                            a[k][j] += f*a[k][i];
                    }
                }
                for (j = i; j < m; j++)
                    a[j][i] *= g;
            }
            else
            {
                for (j = i; j<m; j++)
                    a[j][i] = 0.0;
            }
            a[i][i] += 1.0;
        }
        for (k = n-1; k>=0; --k)
        {
            for (its = 1; its<=30; ++its)
            {
                flag = true;
                for (l = k; l>=0;  --l)
                {
                    nm = l-1;
                    if ((abs(rv1[l])+anorm) == anorm)
                    {
                        flag = false;
                        break ;
                    }
                    if ((abs(w[nm])+anorm)  == anorm) break;
                }
                if (flag)
                {
                    c = 0.0;
                    s = 1.0;
                    for (i = l; i<=k; i++)
                    {	
                        f = s*rv1[i];
                        rv1[i] = c*rv1[i];
                        if ((abs(f)+anorm)==anorm)
                            break;
                        g = w[i];
                        h = pythag(f,g) ;
                        w[i] = h;
                        h = 1.0/h;
                        c = g*h;
                        s = -f*h;
                        for (j = 0; j<m; j++)
                        {
                            y = a[j][nm] ;
                            z = a[j][i];
                            a[j][nm] = y*c+z*s;
                            a[j][i] = z*c-y*s;
                        }
                    }
                } 
                z = w[k];
                if (l==k)
                {
                    if (z<0.)
                    {
                        w[k] = -z;
                        for (j = 0; j<n; j++)
                            v[j][k] = -v[j][k];
                    }
                    break;
                } 
                x = w[l];
                nm = k-1;
                y = w[nm];
                g = rv1[nm] ;
                h = rv1[k] ;
                f = ((y-z)*(y+z)+(g-h)*(g+h))/(2*h*y);
                g = pythag(f,1.0) ;
                f = ((x-z)*(x+z)+h*((y/(f+sign(g,f)))-h))/x;
                c = s = 1.0;
                for (j = l; j<=nm; j++)
                {
                    i = j+1;
                    g = rv1[i];
                    y = w[i];
                    h = s*g;
                    g = c*g;
                    z = pythag(f,h) ;
                    rv1[j] = z;
                    c = f/z;
                    s = h/z;
                    f = x*c+g*s;
                    g = g*c-x*s;
                    h = y*s;
                    y *= c;
                    for (jj = 0; jj<n; jj++)
                    {
                        x = v[jj][j];
                        z = v[jj][i];
                        v[jj][j] = x*c+z*s;
                        v[jj][i] = z*c-x*s;
                    }
                    z = pythag(f,h) ;
                    w[j] = z;
                    if (z != 0.0)
                    {
                        z = 1.0/z;
                        c = f*z;
                        s = h*z;
                    }
                    f = c*g+s*y;
                    x = c*y-s*g;
                    for (jj = 0; jj<m; ++jj)
                    {
                        y = a[jj][j];
                        z = a[jj][i];
                        a[jj][j] = y*c+z*s;
                        a[jj][i] = z*c-y*s;
                    }
                } 
                rv1[l] = 0.0;
                rv1[k] = f;
                w[k] = x;
            } 
        }
        delete rv1;
}

/* Back substitutes U, W, V into original matrix to obtain solution and returns it. */
double* SVD::svbksb(double **u, double *w, double **v, int rows, int cols, double *b) {
	int jj, j, i, m = rows, n = cols;
        double s, *x, *tmp;
        
        tmp = new double[n];
        x = new double[n];
        
        /* Computes U^T*b */
        for (j = 0; j < n; j++) {
        	s = 0.0;
        	/* Nonzero result only if Wj is nonzero. */
            	if (w[j] != 0) {
                	for (i = 0; i < m; i++)
                    		s += u[i][j] * b[i];
                
                	/* This is the divide by Wj */
                	s = s / w[j]; 
            	}
            	tmp[j] = s;
        }
        
        /* Multuplies matrix by V to compute the answer. */
        for (j = 0; j < n; j++) {
            	s = 0.0;
            	for (jj = 0; jj < n; jj++)
                	s += v[j][jj] * tmp[jj];
            
            	x[j] = s;
        }
        
        delete tmp;
        return x;
}
   
/* Zeros any values in the given array of doubles which are negligable according to the given
 * threshold, and return the number of negligable values. */     
int SVD::zeroNegligables(double *m, int length, double threshold) {
	int i, negligables = 0; 
	
	for (i = 0; i < length; i++) {
		if (m[i] < threshold) {
			m[i] = 0.0;
			negligables++; 
		} 
	}
	
	return negligables; 
}

/* Computes the pythagora value given the arguments. */
double SVD::pythag(double a, double b) {
	return sqrt(pow(a , 2) + pow(b , 2));
}

/* Returns the sign of the arguments. */
double SVD::sign(double a, double b) {
	return b >= 0.0 ? abs(a) : -abs(a);
}

/* Returns the absolute value of the argument. */
double SVD::abs(double a) {
	return fabs(a);
}
