
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "Statistics.h"

/* Class constructor. */
Statistics::Statistics() {

}

/* Class constructor with parametes. */
Statistics::Statistics(double avg, double best, double worst, int fitnessCalls) :
	_fitnessCalls(fitnessCalls),
	_avgFitness(avg),
	_bestInstance(best),
	_worstInstance(worst) {

}

/* Class destructor. */
Statistics::~Statistics() {
	delete _bestIndividual;
}	

/* Returns the average population fitness. */
double Statistics::avgFitness() {
	return _avgFitness; 
}

/* Returns the fitness of the best individual. */     
double Statistics::bestInstance() {
	return _bestInstance; 
}

/* Returns the fitness of the worst individual. */
double Statistics::worstInstance() {
	return _worstInstance; 
}

/* Returns the generation number. */
unsigned int Statistics::generation() {
	return _generation; 
}

/* Returns the number of fitness calls. */
unsigned int Statistics::fitnessCalls() {
	return _fitnessCalls; 
}

/* Returns a reference to the best individual instance. */
BinaryInstance* Statistics::bestIndividual() {
	return _bestIndividual;
}

/* Sets the average population fitness value. */
void Statistics::setAvgFitness(double avg) {
	_avgFitness = avg; 
}

/* Sets the best individual fitness value. */
void Statistics::setBestInstance(double best) {
	_bestInstance = best; 
}

/* Sets the worst individual fitness value. */
void Statistics::setWorstInstance(double worst) {
	_worstInstance = worst;
}

/* Sets the generation number. (Perfomed by the Solver object). */
void Statistics::setGeneration(unsigned int generation) {
	_generation = generation; 
}

/* Sets the number of fitness calls. */
void Statistics::setFitnessCalls(unsigned int fitnessCalls) {
	_fitnessCalls = fitnessCalls; 
}

/* Sets the best individual instance. (pass a copy as parameter). */
void Statistics::setBestIndividual(BinaryInstance *best) {
	_bestIndividual = best;
}
