
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/
 
#ifndef COMIT_H_
#define COMIT_H_

#include <iostream> 
#include <stdio.h>
#include <vector>
#include <map>
#include <math.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

#include "Comparator.h"
#include "EDA.h"

using namespace std;

/* This class defines an abstract class for the COMIT algorithm (). It implements the estimation and sampling part 
 * of the algorithm, and lets the selection and fast search operators to concrete objects. */

class COMIT : public EDA {
	
	/* Bayesian network node. */
	class BNN {
		int _index; 
		int _value;
		vector<int> _sons;
		
		public: BNN(int index) { _index = index; };
			virtual ~BNN() { };
			
			vector<int> getSon() { return _sons; };
			void addSon(int son_idx) { _sons.push_back(son_idx); };
			void setValue(int value) { _value = value; };
			int getValue() {return _value; };
			int getIndex() {return _index; };
	};
	
	map<int, BNN*> _bayesian_map;	
	BNN* _bnn_root; 
	
	protected: 
		unsigned int _samples;
		unsigned int _max_influence; 
		vector< BinaryInstance*> _sample_vec;
		vector< BinaryInstance* > _new_fast_search;

	public:
		COMIT(unsigned int popSize, unsigned int samples, unsigned int max_influence, const gsl_rng* rnd);
		virtual ~COMIT();
		
		virtual void selection() = 0;
		virtual void fastsearch() = 0;
		
		void sampling();
		void estimation();
		void replace();
		
		void run();
		void setObjectiveFunction(ObjectiveFunction* of);
	
	private:
		double mutualInformation(int idx1, int idx2); 
		void recursiveSampling(BinaryInstance *instance, int root_idx);
		vector<double> conditionalProbabilities(vector<int> idx, int root_idx, int root_value);
	
};
#endif
