 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)       *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/
 
#ifndef EDA_H_
#define EDA_H_

#include <math.h>
#include <algorithm>

#include "Algorithm.h"
#include "Comparator.h"
#include "Statistics.h"

using namespace std;

/* This class defines a super-abstract class for those algorithms coming from Estimation of Distribution 
 * Algorithms. Every EDA algorithms need to inherit from this abstract class, and to implement the virtual
 * methods. It defines several common parameters, and it implements some common functions. 		
 */

class EDA : public Algorithm {
	protected:
		unsigned int _popSize;			/* Population size. 						*/
		const gsl_rng* _rnd;			/* Random number generator reference.				*/
		vector< BinaryInstance* > _pop;		/* Population container.					*/
		bool parent_;				/* DEPRECATED: TO BE REMOVED, USED BY GEDAs!!!!			*/
		
	public:
		EDA(unsigned int popSize, const gsl_rng* rnd);
		EDA(const EDA &eda);
		virtual ~EDA();
		virtual void sampling() = 0;
		virtual void selection() = 0;
		virtual void estimation() = 0;
		virtual void run();
		
		void printPopulation();
		void printProbVector();
		
		Statistics* computeStatistics();
		
		/* DEPRECATED: TO BE REMOVED, USED BY GEDAs!!!!			*/		
		void setIsParent(bool parent) { parent_ = parent; };
		bool getIsParent() {return parent_; };		
};
#endif
