 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#ifndef UNIVARIATEEDA_H_
#define UNIVARIATEEDA_H_

#include "EDA.h"
#include "Algorithm.h"

using namespace std;

/* This class defines an abstract superclass for those algorithms based on an univariate probabilistic
 * model, represented as a marginal probability vector. It implements common functions as sampling or
 * selection and let to the concrete class to implement estimation operator and, if needed, to overwrite
 * the provided function implementations. */

class UnivariateEDA : public EDA {

	protected:
		vector<double> _prob;

	public:
		UnivariateEDA(unsigned int popSize, const gsl_rng* rnd);
		UnivariateEDA(unsigned int popSize, vector<double> prob, const gsl_rng* rnd);
		virtual ~UnivariateEDA();

		void setObjectiveFunction(ObjectiveFunction* of);
		void sampling();
		void selection();
		
		virtual void estimation() = 0;

		vector<double> getProbabilityVector();
		void setProbabilityVector(vector<double> prob);
		void printJointProbalityFunction();
		void printJointProbalityFunctionDec();
};
#endif
