 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "cGA.h"

/* Class constructor. */
cGA::cGA(unsigned int popSize, const gsl_rng* rnd) : UnivariateEDA(popSize, rnd) {

}

/* Class destructor. */
cGA::~cGA() {
	
}

/* This function defines the single generation algorithm step. */
void cGA::run() {
	sampling();
	selection();	
	estimation();	
}

/* Samples two individual from the univariate distribution. */
void cGA::sampling() {
	for (unsigned int i=0; i<_pop.size(); i++) {
		delete _pop[i];
	}
	_pop.clear();
	for (unsigned int i=0; i<2; i++) {
		BinaryInstance *instance = new BinaryInstance(_prob.size());
		for (unsigned int j=0; j<_prob.size(); j++) {
			instance->set(j,gsl_ran_bernoulli(_rnd, _prob[j]));
		}
		_pop.push_back(instance);
	}
}

/* Does not implements a real selection, such cGA does not need it, but simply order the population. */
void cGA::selection() {
	std::sort(_pop.begin(),_pop.end(),Comparator(this));
}

/* Updates the probability vector according to the current population. */
void cGA::estimation() {
	BinaryInstance *winner = _pop[0];
	BinaryInstance *competer = _pop[1];
	for (unsigned int j=0; j<_prob.size(); j++) {
		double delta = 0;
		if (winner->get(j)!=competer->get(j)) {
			if (winner->get(j)==0)
				delta = -1.0/_popSize;
			else
				delta = 1.0/_popSize;
		}

		_prob[j] = _prob[j] + delta;
		if (_prob[j]>1)
			_prob[j] = 1;
		if (_prob[j]<0)
			_prob[j] = 0;
	}
}

/* Returns the algorithm name. */
string cGA::getName() {
	return "cGA";
}
