
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "hcCOMIT.h"
#include <algorithm>

/* Class constructor. */
hcCOMIT::hcCOMIT(unsigned int popSize, unsigned int samples, unsigned int max_influence, unsigned int patience, double mut_rate, const gsl_rng* rnd) :
	COMIT(popSize, samples, max_influence, rnd),
	_mut_rate(mut_rate),
	_patience(patience) {

}

/* Class destructor. */
hcCOMIT::~hcCOMIT() {

}

/* Selection operator. */
void hcCOMIT::selection() {
	std::sort(_sample_vec.begin(), _sample_vec.end(), Comparator(this));
	_new_fast_search.clear();
	_new_fast_search.push_back(_sample_vec[0]);
	
	for (unsigned int i=1; i<_sample_vec.size(); i++)
		delete _sample_vec[i];
	if (_sample_vec.size() != 0)	_sample_vec.clear(); 
} 

/* Hill-climber fast-search operator. */
void hcCOMIT::fastsearch() {
	int iteration = 0;
	BinaryInstance* best = _new_fast_search[0];
	if (best==NULL) cout << "best is null!";
	_new_fast_search.clear();
	
	while (iteration < _patience) {
		/* Delete old individuals from vector except best */
		if (_new_fast_search.size() > 1) {
			for (unsigned int i=1 ; i< _new_fast_search.size(); i++)
				delete _new_fast_search[i];
			_new_fast_search.clear();
		}

		
		/* Create new individual by mutation of the best */
		for (unsigned int i=0; i<_max_influence; i++)
			_new_fast_search.push_back(mutate(best));
		
		std::sort(_new_fast_search.begin(), _new_fast_search.end(), Comparator(this));
		
		/* Find the new best individual */
		if (fitness(_new_fast_search[0]) > fitness(best)) {
			delete best;                     // delete old-best
			best = _new_fast_search[0];      // set new-best
		} else {
			iteration++;
		}
	} 
}

/* Create a new individual from a mutation of the best one */
BinaryInstance* hcCOMIT::mutate(BinaryInstance* best_individual) {
	BinaryInstance* new_instance = new BinaryInstance(objectiveFunctionSize_);
	
	for (unsigned int j=0; j<objectiveFunctionSize_; j++) {
		if (gsl_ran_bernoulli(_rnd, _mut_rate)) {
			if (best_individual->get(j)==0) 
				new_instance->set(j,1);
			else
				new_instance->set(j,0);
		} else {
			new_instance->set(j, best_individual->get(j));
		}
	}
	
	return new_instance; 
}

/* Returns the algorithm name. */
string hcCOMIT::getName() {
	return "hcCOMIT";
}
