 
 /*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/
 
#include "GA.h"
#include "Comparator.h"

/* Class constructor, initializes the class parameters and checks the parity of the population size. */
GA::GA(unsigned int popSize, double mutRate, const gsl_rng* rnd) : Algorithm() {
	if (popSize % 2 == 1) popSize++;
	_popSize = popSize;
	_mutRate = mutRate;	
	_rnd = rnd;
}

/* Class destructor, it deletes the current population. */
GA::~GA() {
	for (unsigned int i=0; i<_pop.size(); i++) delete _pop[i];
}

/* Sets the objective function for the execution and initializes a random population. */
void GA::setObjectiveFunction(ObjectiveFunction* of) {
	Algorithm::setObjectiveFunction(of);

	for (unsigned int i=0;i<_popSize;i++) {
		BinaryInstance* instance = new BinaryInstance(objectiveFunctionSize_);
		for (unsigned int j=0;j<objectiveFunctionSize_;j++) {
			instance->set(j,gsl_ran_bernoulli(_rnd, 0.5));
		}
		_pop.push_back(instance);
	}
}

/* Printout the individuals population with fitness value. */
void GA::printPopulation() {
	for (unsigned int i=0;i<_pop.size();i++) {
		BinaryInstance* instance = _pop[i];
		for (unsigned int j=0;j<objectiveFunctionSize_;j++) {
			cout << instance->get(j);
		}
		printf(" (%.2f) \n",instance->getFitnessCache());
	}
}

/* Computes algorithm statistics: so far, best and worst instance, average population fitness, 
 * fitness calls counting. */
Statistics* GA::computeStatistics() {
	int bindex = 0;
	double avg = 0.0f, best = -1.0, worst = 101.0, tmp;
	Statistics *stat;
	
	for (unsigned int i=0;i<_pop.size();i++) {
			BinaryInstance* instance = _pop[i];
			tmp = instance->validCache() ? instance->getFitnessCache() : fitness(instance);
			avg += tmp;
			if (tmp > best) { 
				best = tmp;
				bindex = i;
			}
			if (tmp < worst) worst = tmp;
	}
	avg /= _pop.size();
	stat = new Statistics(avg, best, worst, fitnessCalls());
	stat->setBestIndividual(new BinaryInstance(*_pop[bindex]));
	
	return stat;
}
