
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it)	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>.    	 *	
 *********************************************************************************/

#include "SGAPopWiseCross.h"

using namespace std;

/* Class constructor. */
SGAPopWiseCross::SGAPopWiseCross(int popSize, double mutRate, const gsl_rng* rnd) : SGA(popSize, mutRate, rnd) {	

}

/* Class destructor */
SGAPopWiseCross::~SGAPopWiseCross() {

}

/* This function defines the generation step. */
void SGAPopWiseCross::run() {
	selection();
	crossover();
	mutation();
}

/* Uniform bitwise crossover, it doubles the population size by generating new offspring. It assumes, 
 * selection of 50% of the original population. */
void SGAPopWiseCross::crossover() {
        vector<BinaryInstance*> new_pop;
        BinaryInstance *selected, *target;
        int target_value, swap_target;
        unsigned int n_bits = _pop[0]->getSize();
        
        
        //Make a copy of the selected population
        for (unsigned int i=0; i<_pop.size(); i++) {
        	BinaryInstance *instance = new BinaryInstance(*_pop[i]);
        	new_pop.push_back(instance);
        }
        
        //Perform Population Wise Crossover
        for (unsigned int i=0; i<n_bits; i++) {
        	for (unsigned int j=0; j<_pop.size(); j++) {
        		swap_target = gsl_rng_uniform_int(_rnd,_pop.size());
        		
        		selected = new_pop[j];
        		target = new_pop[swap_target];
        		target_value = target->get(i);
        		
        		target->set(i, selected->get(i));
        		selected->set(i, target_value);
        	}
        }
        
        //New population = old pop + crossed pop
        _pop.insert( _pop.end(), new_pop.begin(), new_pop.end() );
}

/* Returns the algorithm name. */
string SGAPopWiseCross::getName() {
	return "SGAPopWiseUniformCrossover";
}
