 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#ifndef ALGORITHMDECORATOR_H
#define ALGORITHMDECORATOR_H

#include <gtkmm.h>
#include <gsl/gsl_rng.h>
#include <vector>
#include <sstream>

#include "Algorithm.h"
#include "Solver.h"
#include "path.h"
#include "Evoptool.h"

using namespace std;

/* This abstract class defines the Algorithm Decorator hierarchy, and it is the superclass of this hierarchy.  
 * Algorithm decorators are gtkmm widgets designed as parameter input for the algorithms. This class implements
 * several functions in order to minimize the effort during a decorator implementations. */

class AlgorithmDecorator : public Gtk::HBox {
	Gtk::CheckButton *_ck_btn;
	Gtk::VBox *_vbox;
	vector<Gtk::HBox *> _hbox; 
	vector<Gtk::Label*> _label;
	Glib::ustring _title;
	
protected:
	Gtk::Expander *_expander;
	vector<Gtk::SpinButton*> _sp_btns;
	enum ParamType {POP, GEDA_POP, EDA_POP, MUT_RATE, GAMMA, FACTOR, EPSILON,
	   		MAX_INCOMING, PARENT_PERCENTAGE, OFFSPRING, NUM_SAMPLES,
	   		PERCENTAGE_SELECTION, MAX_INFLUENCE, FAST_SEARCH_ITERATIONS,
	   		COOLING_RATE, DEPTH, CROSS_RATE, CHAIN_OFFSET, MAX_CLIQUE_SIZE, 
	   		MAX_NUM_OF_CLIQUE};

public:
        AlgorithmDecorator(Glib::ustring &title);
        virtual ~AlgorithmDecorator();
        
        enum PlotType {AVG, BEST, WORST, FCALLS};
        
	virtual Solver* newInstance(Evoptool::TestParameters *params) = 0;
	
	bool isActive();
	void setSensitive(bool mode);

private:
	virtual void initContent() = 0;          /* Initialize the customized part of the widget */
	void initialize(); 

protected:
	void addParameter(ParamType type);

};

#endif
