 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "UMDADecorator.h"
#include "UMDA.h"

/* Class constructor. */
UMDADecorator::UMDADecorator(Glib::ustring &title) : AlgorithmDecorator(title) {
	initContent(); 
}

/* Class destructor. */
UMDADecorator::~UMDADecorator() {

}

/* Initializer of the input entries of the decorator. */
void UMDADecorator::initContent() {
	addParameter(POP);                 
}

/* Returns a Solver object, which is a wrapper around the underlying algorithm. */
Solver* UMDADecorator::newInstance(Evoptool::TestParameters *params) {
	Algorithm **algos;
	Solver *solver;
	unsigned int i, pop;
	ObjectiveFunction *task = params->task;
	unsigned int algorithmsNum = params->averageSamples;
	gsl_rng *r = params->rng; 
	
	pop = _sp_btns[0]->get_value_as_int();      
	
	algos  = new Algorithm*[algorithmsNum];
	for (i = 0; i < algorithmsNum; i++) {
		algos[i] = new UMDA(pop, r);
		algos[i]->setObjectiveFunction(task);
	}
		
	solver = new Solver(algos, algorithmsNum, UMDAFN);
	return solver;
}
