 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "hcCOMITDecorator.h"
#include "hcCOMIT.h"

/* Class constructor. */
hcCOMITDecorator::hcCOMITDecorator(Glib::ustring &title) : AlgorithmDecorator(title) {
	initContent(); 
}

/* Class destructor. */
hcCOMITDecorator::~hcCOMITDecorator() {

}

/* Initializer of the input entries of the decorator. */
void hcCOMITDecorator::initContent() {
	addParameter(POP);
	addParameter(NUM_SAMPLES);
	addParameter(MAX_INFLUENCE);
	addParameter(FAST_SEARCH_ITERATIONS);
	addParameter(MUT_RATE);
}

/* Returns a Solver object, which is a wrapper around the underlying algorithm. */
Solver* hcCOMITDecorator::newInstance(Evoptool::TestParameters *params) {
	Algorithm **algos;
	Solver *solver;
	unsigned int i, pop, samples, maxInfluence, iterations;
	double mutRate;
	ObjectiveFunction *task = params->task;
	unsigned int algorithmsNum = params->averageSamples;
	gsl_rng *r = params->rng;
	
	pop = _sp_btns[0]->get_value_as_int(); 
	samples = _sp_btns[1]->get_value_as_int();
	maxInfluence = _sp_btns[2]->get_value_as_int();
	iterations = _sp_btns[3]->get_value_as_int();
	mutRate = _sp_btns[4]->get_value();
	
	algos  = new Algorithm*[algorithmsNum];
	for (i = 0; i < algorithmsNum; i++) {
		algos[i] = new hcCOMIT(pop, samples, maxInfluence, iterations, mutRate, r);
		algos[i]->setObjectiveFunction(task);
	}
		
	solver = new Solver(algos, algorithmsNum, HCCOMIT);
	return solver;
}
