#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include <glibmm/thread.h>
#include <gtkmm.h>
#include <libglademm.h>
#include <time.h>

#include "Evoptool.h"
#include "EvoptoolGUI.h"
#include "path.h"

/* Prints usage information for this program to STREAM (typically stdout or stderr), and exit the program with
   EXIT_CODE. Does not return. */ 
void print_usage (FILE* stream, int exit_code) {
	fprintf (stream, "Usage: evoptool [options]\n");
	fprintf (stream,
        	" -h --help               		Display this usage information.\n"
        	" -n --nogui [configuration file]   	Execute program from command line with configuration file.\n"
        	" -v --version            		Print software version.\n");
	exit (exit_code);
}

/* Main program entry point. ARGC contains number of argument list elements; ARGV is an array of pointers to them. */
int main (int argc, char* argv[]) {
	int next_option, ret_value = 0;
	bool gui = true; 
	const char* const short_options = "hn:v";
	const struct option long_options[] = {
		{ "help",     0, NULL, 'h' },
    		{ "nogui",   1, NULL, 'n' },
    		{ "version", 0, NULL, 'v' },
                { NULL,       0, NULL, 0   }   
        };
        
   
        do {
        	 next_option = getopt_long (argc, argv, short_options, long_options, NULL);
                 switch (next_option) {
                 	case 'h':   /* -h or --help */
                   		print_usage (stdout, 0);
                   		break; 
                   		
                 	case 'n':   /* -n or --nogui */
                   		/* This option takes an argument, the name of the configuration file.  */
                   		gui = false; 
                   		break;
                   		
                 	case 'v':   /* -v or  --version */
                   		fprintf(stderr, "\n Evoptool: Evolutionary Optimization Tool version 1.0 (2010).\n");
                   		fprintf(stderr, "\n Evptool is free software: you can redistribute it and/or modify \n it under the terms of the GNU General Public License as published by \n the Free Software Foundation, either version 3 of the License, or \n (at your option) any later version. \n \n Evoptool is distributed in the hope that it will be useful, \n but WITHOUT ANY WARRANTY; without even the implied warranty of \n MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n GNU General Public License for more details.	\n \n You should have received a copy of the GNU General Public License \n along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. \n \n ");
                   		fprintf(stderr, "See http://airwiki.elet.polimi.it/mediawiki/index.php/Evoptool:_Evolutionary_Optimization_Tool for further informations.\n \n");
                   		exit(0);
                   		break;
                   		
                 	case '?':   /* The user specified an invalid option. */
                   		fprintf(stderr, "Evoptool: invalid option specified!\n");
                   		exit(1);
                   		
                 	case -1:    /* Done with options.  */
                 		Evoptool *tool;
                 		
                 		if (gui) {
                 			if(!Glib::thread_supported()) Glib::thread_init();
                 			Glib::RefPtr<Gnome::Glade::Xml> refXml;
                 			Gtk::Main gui(argc, argv);
                 			EvoptoolGUI *toolGUI;
                 			
                 			tool = new Evoptool(Evoptool::GUI);
                 			tool->init();
                 			
                 			refXml = Gnome::Glade::Xml::create(GLADE_PATH);
					refXml->get_widget_derived("gui", toolGUI);
                 			toolGUI->setEvoptoolReference(tool);
                 			
                 			gui.run(*toolGUI);
                 			delete tool;
                 			ret_value = 0;
                 		} else {
                 			tool = new Evoptool(Evoptool::COMMAND_LINE);
                 			
                 			tool->init();
                 			tool->execute(argv[2]);
                 			delete tool;
                 			ret_value = 0;
                 		}
                   		break;
                   		
                 	default:    /* Something else: unexpected.  */
                   		abort ();
                 }
        } while (next_option != -1);
        
        return ret_value; 
}
