#include <string>
#include <iostream>

#include "sBOADecorator.h"
#include "sBOA.h"
#include "path.h"

sBOADecorator::sBOADecorator(Glib::ustring &title) : AlgorithmDecorator(title) {
	initContent(); 
}

sBOADecorator::~sBOADecorator() {

}

void sBOADecorator::initContent() {
	addParameter(POP);
	addParameter(MAX_INCOMING); 
	addParameter(PARENT_PERCENTAGE);
}

Solver* sBOADecorator::newInstance(Evoptool::TestParameters *params) {
	Solver *solver_; 
	Algorithm **algos;
	string line;
	ofstream conf_file;
	ObjectiveFunction *task = params->task;
	string fileName = "none";
	unsigned int fitness_fun, paramT = 0;
	
	if( dynamic_cast< OneMax* >(task) != 0 )	
		fitness_fun = 0;
	else if( dynamic_cast< Quadratic* >(task) != 0 ) 
		fitness_fun = 1;
	else if( dynamic_cast< F3Deceptive* >(task) != 0 ) 
		fitness_fun = 2;
	else if( dynamic_cast< Trap5* >(task) != 0 ) 
		fitness_fun = 3;
	else if( dynamic_cast< F3DeceptiveBipolar* >(task) != 0 ) 
		fitness_fun = 4;
	else if( dynamic_cast< F3DeceptiveOverlapping* >(task) != 0 ) 
		fitness_fun = 5;
	else if( dynamic_cast< AltBits* >(task) != 0 ) 
		fitness_fun = 6;
	else if( dynamic_cast< BinVal* >(task) != 0 ) 
		fitness_fun = 7;
	else if( dynamic_cast< LiepinsVoseFullyDeceptive* >(task) != 0 ) 
		fitness_fun = 8;
	else if( dynamic_cast< OneZeroMax* >(task) != 0 ) 
		fitness_fun = 9;
	else if( dynamic_cast< SumVal* >(task) != 0 ) 
		fitness_fun = 10;
	else if( dynamic_cast< FourPeaks* >(task) != 0 ) { 
		fitness_fun = 11;
		paramT = dynamic_cast< FourPeaks* >(task)->paramT();
	}
	else if( dynamic_cast< SixPeaks* >(task) != 0 ) { 
		fitness_fun = 12;
		paramT = dynamic_cast< SixPeaks* >(task)->paramT();
	}
	else if( dynamic_cast< SpinGlass* >(task) != 0 ) { 
		fitness_fun = 13;
		fileName = dynamic_cast< SpinGlass* >(task)->fileName();
	}
	else if( dynamic_cast< SatLib* >(task) != 0 ) { 
		fitness_fun = 14;
		fileName = dynamic_cast< SatLib* >(task)->fileName();
	}
	else if( dynamic_cast< TextureRestoration* >(task) != 0 ) {
		fitness_fun = 15;
		fileName = dynamic_cast< TextureRestoration* >(task)->fileName();
		}
	else
		fitness_fun = 0; /* default option */
	
	conf_file.open(SBOA_CONF_FILE_EV);

	conf_file << "populationSize = " << _sp_btns[0]->get_value_as_int() << endl;
	conf_file << "problemSize = " << task->getSize() << endl;
	conf_file << "fitnessFunction = " << fitness_fun << endl;
	conf_file << "parentsPercentage = " << _sp_btns[2]->get_value_as_int() << endl;
	conf_file << "offspringPercentage = " << 100 - _sp_btns[2]->get_value_as_int() << endl;
	conf_file << "maxIncoming = " << _sp_btns[1]->get_value_as_int() << endl;
	conf_file << "maxNumberOfGenerations = " << params->maxGenerations << endl;
	conf_file << "epsilon = " << -1 << endl;
	conf_file << "outputFile = sBOA.dat" << endl;
	conf_file << "randSeed = 314159265" << endl;
	conf_file << "maxFitness = 100.0" << endl;
	conf_file << "paramT = " << paramT << endl;
	conf_file << "fitnessFile = " << fileName << endl;

	conf_file.close();
	
	
	algos  = new Algorithm*[1];
	algos[0] = new sBOA(SBOA_CONF_FILE);
	algos[0]->setObjectiveFunction(task);	
	solver_ = new Solver(algos, 1, "f2_sboa.txt");

	return solver_; 
}
