#include <math.h>
 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "LiepinsVoseFullyDeceptive.h"

/* Class constructor. */
LiepinsVoseFullyDeceptive::LiepinsVoseFullyDeceptive(int size) : ObjectiveFunction(size) {

}

/* Class destructor. */
LiepinsVoseFullyDeceptive::~LiepinsVoseFullyDeceptive() {

}

/* Fitness function. */
double LiepinsVoseFullyDeceptive::f(BinaryInstance *bi) {
	double volatile returnValue;
	if (bi != NULL) {
		if(!(bi->validCache())) { 
			/* Fitness cache value not valid */
			int ones = countOnes(bi);
			if (ones == _size) 
				returnValue = 1 * 100;
			else if (ones == 0) 
				returnValue = (1 - 1.0/ pow(2,_size)) * 100;
			else			
				returnValue = (1 - (1.0 + (double)ones) / (_size)) * 100;
			bi->setFitnessCache(returnValue);
			return returnValue;
		} else {
			return bi->getFitnessCache();
		}
	} else {
		cerr << "[LiepinsVoseFullyDeceptive::f] binary instance cannot be null" << endl;
		return 0;
	}
}

/* Reterns the count of bits taking value one. */
int LiepinsVoseFullyDeceptive::countOnes(BinaryInstance *bi) {
	int sum = 0;
	for (int j=0; j<_size; j++) {
		sum = sum + bi->get(j);
	}
	return sum;
}

