 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "Quadratic.h"

/* Class constructor. */
Quadratic::Quadratic(int size) : ObjectiveFunction(size) {

}

/* Class destructor. */
Quadratic::~Quadratic() {

}

/* Fitness function. */
double Quadratic::f(BinaryInstance *bi) {
	double normalize;
	
	if (_size % 2 == 0) normalize = _size / 2;
	else normalize = (_size - 1) / 2;
	
	if (bi!=NULL) {
		if(!(bi->validCache())) { 
			/* Fitness cache value not valid */
			double sum = 0; 
			for (int j=0; j<_size - 1; j=j+2) {
				bool a = bi->get(j);
				bool b = bi->get(j+1);
				if (a==0)
					if (b==0)
						sum += 0.9;
					else
						sum += 0;
				else
					if (b==0)
						sum += 0;
					else
						sum += 1;
			}

			volatile double retValue = ((double)sum) / normalize * 100.0;
			bi->setFitnessCache(retValue);
			return retValue;
		} else {
			return bi->getFitnessCache();
		}
	} else {
		cerr << "[Quadratic::f] binary instance cannot be null" << endl;
		return 0;
	}
}
