 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/


#include "SatLib.h"
#include <stdlib.h> 

/* Class constructor. */
SatLib::SatLib(char* filename) : ObjectiveFunction(1), fileName_(filename) {
	file_.open(filename);
	string oneline;
	int x,y,z,w;
	string token, type;
	while(file_) {
		char h = file_.peek();
 		getline(file_,oneline);
		stringstream ss(oneline);
		if (h=='c' || h=='%' || oneline.length()<2) {
		} else {
			if (h=='p') {
				ss >> token >> type >> x >> y;	
				_size = x;
				nClauses_ = y;
			} else {
				ss >> x >> y >> z >> w;	
				ThreeClause* sat = new ThreeClause(abs(x)-1,abs(y)-1,abs(z)-1,sign(x),sign(y),sign(z));
				clauses_.push_back(sat);
			}
		}
	}
}

/* Class destructor. */
SatLib::~SatLib() {
	file_.close();
	for (unsigned int i=0;i<clauses_.size();i++) {
		delete clauses_[i];
	}
}

/* Fitness function. */
double SatLib::f(BinaryInstance *bi) {
	if (bi!=NULL) {
		if(!(bi->validCache())) { 
			/* Fitness cache value not valid */
			int sum = 0;
			for (unsigned int i=0;i<clauses_.size();i++) {
				if (clauses_[i]->isSatisfied(bi)) {
					sum++;
				}
			}
			volatile double retValue = ((double)sum)/((double)(nClauses_))*100.0;
			bi->setFitnessCache(retValue);
			return retValue;
		} else {
			return bi->getFitnessCache();
		}
	} else {
		cerr << "[SatLib::f] binary instance cannot be null" << endl;
		return 0;
	}
}

/* Returns the filename of this objective function instances. */
string SatLib::fileName() {
	return fileName_;
}


/* Returns true for a given positive parameter, false otherwise. */
bool SatLib::sign(int n) { 
	return n >= 0;
}
