 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#ifndef SATLIB_H
#define SATLIB_H

#include <fstream>
#include <sstream>
#include <string>

#include "ObjectiveFunction.h"

/* Defines the clause of three variables. */
class ThreeClause;			

/* This class defines the Max Sat objective function. Actually, it is an instance of lenght 100 with 
 * 430 clauses created by SatLib. */

class SatLib : public ObjectiveFunction {

private:
	vector< ThreeClause* > clauses_;
	ifstream file_;
	unsigned int nClauses_;
	string fileName_;

public:
        SatLib(char* filename);
        virtual ~SatLib();

	double f(BinaryInstance* bi);
	string fileName();
private: 
	bool sign(int n);
};

/* This class defines the clause of three variables. */
class ThreeClause {

private:
	int iIndex_, jIndex_, kIndex_;
	int iValue_, jValue_, kValue_;

public:
        ThreeClause(int iIndex, int jIndex, int kIndex, bool iValue, bool jValue, bool kValue) : iIndex_(iIndex), jIndex_(jIndex), kIndex_(kIndex), iValue_(iValue), jValue_(jValue), kValue_(kValue) {};
        virtual ~ThreeClause() {};

	bool isSatisfied(BinaryInstance* bi) {
		return ((bi->get(iIndex_)==iValue_) || (bi->get(jIndex_)==jValue_) || (bi->get(kIndex_)==kValue_));
	};
};
#endif
