 
/*********************************************************************************
 *  Copyright© 2010 Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include <math.h>
#include "SixPeaks.h"

/* Class constructor. */
SixPeaks::SixPeaks(int size, int t) : ObjectiveFunction(size) {
	t_ = t > size ? size - 1 : t;
}

/* Class destructor. */
SixPeaks::~SixPeaks() {

}

/* Fitness function. */
double SixPeaks::f(BinaryInstance *bi) {
	if (bi != NULL) {
		if(!(bi->validCache())) { 
			/* Fitness cache value not valid */
			int sum = r(bi);
			sum += max(headingBits(bi,1),tailingBits(bi,0));	
			volatile double retValue = (double) sum / ((double) (_size + _size - t_)) * 100;
			bi->setFitnessCache(retValue);
			return retValue;
		} else {
			return bi->getFitnessCache();
		}
	} else {
		cerr << "[SixPeaks::f] binary instance cannot be null" << endl;
		return 0;
	}
}

/* Returns the parameter T. */
int SixPeaks::paramT() {
	return t_;
}

/* Reward function. */
int SixPeaks::r(BinaryInstance *bi) {
	int rtnValue = 0;
	if ((headingBits(bi,1)>=t_ && tailingBits(bi,0)>=t_) || (headingBits(bi,0)>=t_ && tailingBits(bi,1)>=t_)) {
		rtnValue = _size;
	}
	return rtnValue;
}

/* Returns the number of adjacent bits taking a specified value starting from the begging of the individual. */
int SixPeaks::headingBits(BinaryInstance *bi, bool bit) {
	int sum = 0;
	int j = 0;
	while(j<_size && bi->get(j)==bit) {
		sum++;
		j++;
	}	
	return sum;
}

/* Returns the number of adjacent bits taking a specified value starting from the end of the individual*/
int SixPeaks::tailingBits(BinaryInstance *bi, bool bit) {
	int sum = 0;
	int j = _size-1;
	while(j>=0 && bi->get(j)==bit) {
		sum++;
		j--;
	}	
	return sum;
}
