 
/*********************************************************************************
 *  Copyright© 2010 Gabriele Valentini (gabriele.valentini@mail.polimi.it),	 *
 *                  Luigi Malagò (malago@elet.polimi.it)                         *
 *										 *
 *  This file is part of Evoptool.						 *
 * 										 *
 *  Evptool is free software: you can redistribute it and/or modify              *
 *  it under the terms of the GNU General Public License as published by         *
 *  the Free Software Foundation, either version 3 of the License, or            *
 *  (at your option) any later version.                                          *
 *										 *
 *  Evoptool is distributed in the hope that it will be useful,                  *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  		 *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 		 *
 *  GNU General Public License for more details.				 *
 *										 *
 *  You should have received a copy of the GNU General Public License		 *
 *  along with Evoptool.  If not, see <http://www.gnu.org/licenses/>. 		 *	
 *********************************************************************************/

#include "Algorithm.h"

/* Class constructor. */
Algorithm::Algorithm() {
	_of = NULL;
	objectiveFunctionSize_ = 0;
}

/* Class destructor. */
Algorithm::~Algorithm() {

}

/* Sets the objective function reference for the algorithm, and initializes class parameters fitnessCalls_
 * and objectiveFunctionSize_. */
void Algorithm::setObjectiveFunction(ObjectiveFunction* of) {
	if (of) {
		_of = of;
		fitnessCalls_ = 0;
		objectiveFunctionSize_ = _of->getSize();
	} else {
		cerr << "[Algorithm::setObjectiveFunction] of cannot be null." << endl;
	}
}

/* Set for this algorithm the same objective function used by the algorithm argument passed. */
void Algorithm::setObjectiveFunction(const Algorithm &algo) {
	_of = algo._of;
	fitnessCalls_ = 0;
	objectiveFunctionSize_ = algo._of->getSize();
}

/* Wrapper for the objective function, so we are sure to exploit the fitness value caching 
 * mechanism and to correctly count the number of fitness calls. */
double Algorithm::fitness(BinaryInstance *instance) {
	double rval;
	
	if (instance->validCache())
		return instance->getFitnessCache();
	else {
		rval = _of->f(instance);
		instance->setFitnessCache(rval);
		fitnessCalls_++;
		return rval;
	}
}

/* Returns the amout of fitness function calls. */
unsigned int Algorithm::fitnessCalls() {
	return fitnessCalls_;
}
